//+------------------------------------------------------------------+
//|                       Enhanced version of  Wilder's DMI book.mq4 |
//|                                                  coded by mladen |
//|                                                                  |
//| Directional movement index was developed by Welles Wilder        |
//|                                                                  |
//| Enhanced by "Big Be" Sept 2012   MrPipForex.com                  |
//|                                                                  |
//+------------------------------------------------------------------+

#property copyright "mladen, with changes by Big Be"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property  indicator_buffers 5
#property indicator_minimum  0

#property indicator_color3   Blue   // DimGray
#property indicator_color4   Orange
#property indicator_color5   Blue   // DimGray
#property indicator_width3   2 
#property indicator_width4   2 
#property indicator_style5   STYLE_DOT

//
//
//
//
//

extern int  DMI.Length =    14;
extern bool ShowDIs    = false;
extern bool ShowADX    =  true;
extern bool ShowADXR   = false;
extern int  Level      =    20;

//
//
//
//
//

double DIp[];
double DIm[];
double ADX[];
double ADXR[];
double ADXLevel[];
double sumDIp[];
double sumDIm[];
double sumTR[];

double sf;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,DIp);      SetIndexLabel(0,"DI+");
   SetIndexBuffer(1,DIm);      SetIndexLabel(1,"DI-");
   SetIndexBuffer(2,ADX);      SetIndexLabel(2,"ADX");
   SetIndexBuffer(3,ADXR);     SetIndexLabel(3,"ADXR");
   SetIndexBuffer(4,ADXLevel); SetIndexLabel(4,NULL);
   SetIndexBuffer(5,sumDIp);
   SetIndexBuffer(6,sumDIm);
   SetIndexBuffer(7,sumTR);
   
   for (int i=0;i<8;i++) SetIndexEmptyValue(i,0.00);

   //
   //
   //
   //
   //
   
   if(ShowDIs)
   {
   SetIndexStyle(0, DRAW_LINE, STYLE_DOT, 1, Lime);
   SetIndexStyle(1, DRAW_LINE, STYLE_DOT, 1, Tomato);
   }
   else
   {
   SetIndexStyle(0,DRAW_NONE);   SetIndexLabel(0,NULL);
   SetIndexStyle(1,DRAW_NONE);   SetIndexLabel(1,NULL);
   }
   
   if(ShowADX == false) SetIndexStyle(2,DRAW_NONE);
   else SetIndexStyle(2,DRAW_LINE);  
   
   if(ShowADXR == false) SetIndexLabel(3,NULL);
   
   sf = (DMI.Length-1.0)/DMI.Length;   // sum factor
   
   IndicatorShortName("ADX Wilder\'s Book ("+DMI.Length+")");
   return(0);
}

int deinit()
{
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
       limit=Bars-counted_bars;
       limit = MathMin(limit, Bars - DMI.Length-1);

   //
   //
   //
   //
   //

      for (i=limit;i>=0;i--)
      {
         double currTR  = MathMax(High[i],Close[i+1])-MathMin(Low[i],Close[i+1]);
         double DeltaHi = High[i] - High[i+1];
	      double DeltaLo = Low[i+1] - Low[i];
         double plusDM  = 0.00;
         double minusDM = 0.00;
         
            if ((DeltaHi > DeltaLo) && (DeltaHi > 0)) plusDM  = DeltaHi;
            if ((DeltaLo > DeltaHi) && (DeltaLo > 0)) minusDM = DeltaLo;      
         
         //
         //
         //
         //
         //
         
            sumDIp[i] = sf *sumDIp[i+1] + plusDM;
            sumDIm[i] = sf *sumDIm[i+1] + minusDM;
            sumTR[i]  = sf *sumTR[i+1]  + currTR;
            ADXLevel[i]   = Level;

         //
         //
         //
         //
         //
                  
            DIp[i] = 0.00;                   
            DIm[i] = 0.00;                   
            if (sumTR[i] > 0)
               {              
                  DIp[i] = 100.00 * sumDIp[i]/sumTR[i];
                  DIm[i] = 100.00 * sumDIm[i]/sumTR[i];
               }            

            if(ShowADX || ShowADXR)
               {
                  double DX;
                  if((DIp[i] + DIm[i])>0) 
                       DX = 100 *MathAbs(DIp[i] - DIm[i])/(DIp[i] + DIm[i]); 
                  else DX = 0.00;
                  ADX[i] = sf *ADX[i+1] + DX/DMI.Length; 
                  
                  if(ShowADXR)
                         ADXR[i] = 0.5*(ADX[i] + ADX[i+DMI.Length]);
               }
      }   
   return(0);      
}